@@ -1,8 +1,8 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
-#  SPDX-License-Identifier: Apache-2.0
-"""
-Python Package for controlling Alexa devices (echo dot, etc) programmatically.
+"""Python Package for controlling Alexa devices (echo dot, etc) programmatically.
+
+SPDX-License-Identifier: Apache-2.0
+
+Websocket library.
 
 For more details about this api, please refer to the documentation at
 https://gitlab.com/keatontaylor/alexapy
