@@ -1,8 +1,8 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
-#  SPDX-License-Identifier: Apache-2.0
-"""
-Python Package for controlling Alexa devices (echo dot, etc) programmatically.
+"""Python Package for controlling Alexa devices (echo dot, etc) programmatically.
+
+SPDX-License-Identifier: Apache-2.0
+
+Constants.
 
 For more details about this api, please refer to the documentation at
 https://gitlab.com/keatontaylor/alexapy
@@ -19,7 +19,7 @@ LOCALE_KEY = {
     ".co.uk": "en_GB",
     ".in": "en_IN",
     ".com": "en_US",
-    ".es": "en_ES",
+    ".es": "es_ES",
     ".mx": "es_MX",
     ".fr": "fr_FR",
     ".it": "it_IT",
