@@ -1,8 +1,6 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
-#  SPDX-License-Identifier: Apache-2.0
-"""
-Python Package for controlling Alexa devices (echo dot, etc) programmatically.
+"""Python Package for controlling Alexa devices (echo dot, etc) programmatically.
+
+SPDX-License-Identifier: Apache-2.0
 
 Helpers.
 
@@ -47,6 +45,11 @@ def hide_email(email: Text) -> Text:
     return hide_serial(email)
 
 
+def hide_password(value: Text) -> Text:
+    """Obfuscate password."""
+    return f"REDACTED {len(value)} CHARS"
+
+
 def hide_serial(item: Optional[Union[dict, str, list]]) -> Union[dict, str, list]:
     """Obfuscate serial."""
     if item is None:
@@ -92,7 +95,7 @@ def obfuscate(item):
         response = item.copy()
         for key, value in item.items():
             if key in ["password"]:
-                response[key] = f"REDACTED {len(value)} CHARS"
+                response[key] = hide_password(value)
             elif key in ["email"]:
                 response[key] = hide_email(value)
             elif key in ["cookies_txt"]:
@@ -158,7 +161,8 @@ def _catch_all_exceptions(func):
                 obfuscate(kwargs),
                 EXCEPTION_TEMPLATE.format(type(ex).__name__, ex.args),
             )
-            login.status["login_successful"] = False
+            if login:
+                login.status["login_successful"] = False
             raise AlexapyLoginError from ex
         except (ContentTypeError) as ex:
             _LOGGER.warning(
@@ -169,7 +173,8 @@ def _catch_all_exceptions(func):
                 obfuscate(kwargs),
                 EXCEPTION_TEMPLATE.format(type(ex).__name__, ex.args),
             )
-            login.status["login_successful"] = False
+            if login:
+                login.status["login_successful"] = False
             raise AlexapyLoginError from ex
         except CancelledError as ex:
             _LOGGER.warning(
